unit sim07u1;

(*For tutorial ((WEBPAGE))-
  Some general tricks and techniques, illustrated along the way
  of simulating a simple woodland's population dynamics.

  Extension of sim(0)06... which had a basic flaw. See the tut
  relating to that before starting to study this, if you want
  to learn as much as possible from the work it will all take.
  *)

(*The current settings for the various rates of Adding and Reducing,
  combined with current scaling and offset constants "work"... make
  a note of them, before tweaking!

  (Better yet: Build in the code to save and load scenarios!)
*)


{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ExtCtrls;

const vers='30 Jan 16';
  wImageW=560;//Width and height of the graph drawing "surface"
  wImageH=160;
  wImageWm1=wImageW-1;
  wImageHm1=wImageH-1;

  kClockEnabled='Stop Auto Advance';//What should be on Start/Stop when clock running
  kClockDisabled='Start Auto Advance';

  //For the graphing, only....
  clGrass=clGreen;
  clRabbit=clBlue;
  clFox=clRed;

  //Scaling and offset of data presentation
  wScGrass=1;//maximum flattening of plotted line.
  wOffGrass=50;
  wScRabbit=100;//considerable flattening of line.
  wOffRabbit=30;
  wScFox=2500;
  wOffFox=2;
  //End of "For graphing..."

type

  { Tsim07f1 }

  Tsim07f1 = class(TForm)
    buStartStop: TButton;
    buRestartSim: TButton;
    buQuit: TButton;
    buDoYear: TButton;
    Image1: TImage;
    laHomepageTxt: TLabel;
    laGrassColor: TLabel;
    laRabbitColor: TLabel;
    laFoxColor: TLabel;
    laMemoColHeadings: TLabel;
    laBM_def: TLabel;
    meReportData: TMemo;
    Timer1: TTimer;
    procedure buQuitClick(Sender: TObject);
    procedure buDoYearClick(Sender: TObject);
    procedure buRestartSimClick(Sender: TObject);
    procedure buStartStopClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    { private declarations }
    Bitmap:TBitmap;
    lwBM_grass, lwBM_rabbit, lwBM_fox:longword;
    lwBM_grassPrev, lwBM_rabbitPrev, lwBM_foxPrev,
    lwToAdd, lwToSubtract:longword;
    wYr,
      wGrassAD,wGrassRD,
      wRabbitAD,wRabbitRD,
      wFoxAD,wFoxRD          :word;
    procedure ReportBiomasses;
    procedure InitOnce;
    procedure InitToRestartSim;
    procedure AddToTopOfMemo(sLineToAddL:string);
    function sLeftPadded(lwToRender:longword;bHowMany:byte):string;
    procedure EstablishInitialRates;

    procedure DoYear;
    procedure MakeNoteOfPreviousBiomasses;
    procedure GrassAdd;
    procedure GrassReduce;
    procedure RabbitsAdd;
    procedure RabbitsReduce;
    procedure FoxesAdd;
    procedure FoxesReduce;

    //START "Just for graph" material
    procedure ClearDrawingSurface;
    procedure PlotPoints;
    procedure PlotPoint(clColorL:TColor;
                 wXL,wScaleY,wOffsetY:word;lwToPlotYL:longword);
    procedure ScrollGraph;
    procedure DrawDiag;
    procedure DrawLine(clL:TColor;wXL0,wYL0,wXL1,wYL1:word);
    //END "Just for graph" material

  public
    { public declarations }
  end;

var
  sim07f1: Tsim07f1;

implementation

{$R *.lfm}

{ Tsim07f1 }

procedure Tsim07f1.DoYear;
begin
  inc(wYr);
  MakeNoteOfPreviousBiomasses;
  GrassAdd;
  GrassReduce;
  RabbitsAdd;
  RabbitsReduce;
  FoxesAdd;
  FoxesReduce;
  ScrollGraph;//Scroll, leaving right hand column empty for data
  ReportBiomasses;//Includes put latest data on graph
end;//DoYear

procedure Tsim07f1.EstablishInitialRates;
//Some of these rates may be static, may remain unchanged
//  over the whole execution of the program's code.
//ppk: Parts per thousand. I.e. 1020 is equivalent to 102 percent
begin
  //Basic rates... in a fancier version of the program, these might
  //   be enriched or made more lean by the influence of other factors,
  //   e.g. the abundance of that organisms food.
  //Abundance of grazers/ predators already affects grass and rabbit
  //   populations, given what the main GrassRD and RabbitRD are
  //   applied to.
  wGrassAD:=30;//"Grass: Added to at rate (Delta) of xx ppk."
                 //(Grass growth per cycle.) e.g. if 20, grass increases 2%
  wGrassRD:=8;//Reduction of grass per cycle, per 100 units of rabbit biomass

  wRabbitAD:=30;//Rabbit increase, ppk. E.g. 40 makes for 4% increase
  wRabbitRD:=15;//Reduction of rabbit per cycle, per 100 units of fox biomass
  wFoxAD:=18;//Fox increase, ppk. E.g. 20 makes for 2% increase
  wFoxRD:=20;//Fox decrease, ppk. E.g. 20 makes for 20% decrease

end;//EstablishInitialRates;

procedure Tsim07f1.GrassAdd;
begin
  lwToAdd:=(lwBM_grassPrev*wGrassAD) div 1000;
  lwBM_grass:=lwBM_grass+lwToAdd;
end;//GrassAdd

procedure Tsim07f1.GrassReduce;
begin
  lwToSubtract:=(lwBM_rabbit*wGrassRD) div 100;
  if (lwBM_grass>lwToSubtract) then lwBM_grass:=lwBM_grass-lwToSubtract//no ; here
    else lwBM_grass:=0;
end;//GrassReduce

procedure Tsim07f1.RabbitsAdd;
begin
  lwToAdd:=(lwBM_rabbitPrev*wRabbitAD) div 1000;
  lwBM_rabbit:=lwBM_rabbit+lwToAdd;
end;//RabbitsAdd

procedure Tsim07f1.RabbitsReduce;
begin
  lwToSubtract:=(lwBM_fox*wRabbitRD) div 100;
  if (lwBM_rabbit>lwToSubtract) then lwBM_rabbit:=lwBM_rabbit-lwToSubtract//no ; here
    else lwBM_rabbit:=0;
end;//RabbitsReduce

procedure Tsim07f1.FoxesAdd;
begin
  lwToAdd:=(lwBM_foxPrev*wFoxAD) div 1000;
  lwBM_fox:=lwBM_fox+lwToAdd;
end;//FoxesAdd

procedure Tsim07f1.FoxesReduce;
begin
  lwToSubtract:=(lwBM_foxPrev*wFoxRD) div 1000;
  if (lwBM_fox>lwToSubtract) then lwBM_fox:=lwBM_fox-lwToSubtract//no ; here
    else lwBM_fox:=0;
end;//FoxesReduce

function Tsim07f1.sLeftPadded(lwToRender:longword;bHowMany:byte):string;
var sTmpL:string;
begin
  sTmpL:=inttostr(lwToRender);
  while NOT(length(sTmpL)>=bHowMany) do
    sTmpL:=' '+sTmpL;
  result:=sTmpL;
end;//sLeftPadded

  procedure Tsim07f1.AddToTopOfMemo(sLineToAddL:string);
var wTmpL:word;
begin
  meReportData.lines.insert(0,sLineToAddL);
end;//of AddToTopOfMemo

procedure Tsim07f1.InitOnce;
begin
 randomize;
 meReportData.lines.clear;
 AddToTopOfMemo('Welcome to sim07');
 AddToTopOfMemo('Version: '+vers);
 AddToTopOfMemo('   dynamics simulation.');
 AddToTopOfMemo('   tutorials, including one discussing this population');
 AddToTopOfMemo('See http://sheepdogguides.com/lut.htm for programming');
 AddToTopOfMemo(' ');
 laMemoColHeadings.caption:=' Year   Grass BM   Rabbit BM   Fox BM';
 laGrassColor.font.color:=clGrass;
 laRabbitColor.font.color:=clRabbit;
 laFoxColor.font.color:=clFox;

end;//InitOnce;

procedure Tsim07f1.InitToRestartSim;
begin
  lwBM_fox:=1000;//Top predator... 1/10th biomass of prey
  lwBM_rabbit:=lwBM_fox*10;//Herbivore.... 1/100th biomass of plants
  lwBM_grass:=lwBM_rabbit*100;

  wYr:=0;
  EstablishInitialRates;
  ClearDrawingSurface;
  ReportBiomasses;
end;//InitToRestartSim;

procedure Tsim07f1.buQuitClick(Sender: TObject);
begin
  close;//terminate doesn't go through "close" event handler
end;

procedure Tsim07f1.ReportBiomasses;
begin
  AddToTopOfMemo(sLeftPadded(wYr,4)+
     sLeftPadded(lwBM_grass,11)+
     sLeftPadded(lwBM_rabbit,11)+
     sLeftPadded(lwBM_fox,10));
  PlotPoints;
end;// ReportBiomasses;

procedure Tsim07f1.MakeNoteOfPreviousBiomasses;
begin
  lwBM_grassPrev:=lwBM_grass;
  lwBM_rabbitPrev:=lwBM_rabbit;
  lwBM_foxPrev:=lwBM_fox;
end;//MakeNoteOfPreviousBiomasses

procedure Tsim07f1.buDoYearClick(Sender: TObject);
begin
  DoYear;
end;

procedure Tsim07f1.buRestartSimClick(Sender: TObject);
begin
  AddToTopOfMemo(' ');
  AddToTopOfMemo(' ');
  AddToTopOfMemo('Simulation restarted...');
  AddToTopOfMemo(' ');
  InitToRestartSim;//Includes prepare drawing surface
  meReportData.VertScrollBar.Position:=0;//"should" work(?) but there's
  //a suggestion in a discussion online...
  //http://forum.lazarus.freepascal.org/index.php?topic=30468.0
  //that maybe this can't be used by most Lazarus people as of late 2015
  //NOT a critical element of the program... only causes problem if you
  //  scroll down after memo gets to big to show in window, and then
  //  do a reset, and aren't willing to rescroll to the top by hand.


end;

procedure Tsim07f1.buStartStopClick(Sender: TObject);
begin
  if buStartStop.caption=kClockDisabled then begin //Turn clock on
      DoYear;
      Timer1.enabled:=true;
      buStartStop.caption:=kClockEnabled;
      end//no ; here
    else begin  //Turn clock off
      Timer1.enabled:=false;
      buStartStop.caption:=kClockDisabled;
      end;
end;

procedure Tsim07f1.FormActivate(Sender: TObject);
begin
  InitOnce;
  InitToRestartSim;//Cannot be done in FormActivate
     //handler, as they affect then not-yet-created
     //controls.
end;

procedure Tsim07f1.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  Bitmap.free;
end;

procedure Tsim07f1.FormCreate(Sender: TObject);
begin
  //Shift running form up and left from were the in-compiler
  //  form is positioned, to make selecting either easy during
  //  development. The app should of course, be given an ini
  //  file which "remembers" the user's preferences.
  top:=top-20;
  left:=left-20;

  sim07f1.caption:='sim07- framework for population dynamics model, and similar. Vers '+ vers;
  application.title:='sim07';
  Timer1.enabled:=false;
  Timer1.interval:=200;
  buStartStop.caption:=kClockDisabled;

  //Set up a non-resizeable "surface" for doing graph on.
  Image1.width:=wImageW;//wImageW, wImageH constants, set at
      //top of code.
  Image1.height:=wImageH;

  Bitmap:=TBitmap.create;
  Bitmap.width:=wImageW;
  Bitmap.height:=wImageH;
  //End of set up "surface"
end;//FormCreate

procedure Tsim07f1.Timer1Timer(Sender: TObject);
begin
  DoYear;
end;

procedure Tsim07f1.ClearDrawingSurface;
begin
  Bitmap.canvas.pen.color:=clWhite;
  Bitmap.canvas.Rectangle
       (0,0,wImageW,wImageH);

  Image1.Picture.Graphic:=Bitmap;
  DrawDiag;
end;//ClearDrawingSurface

procedure Tsim07f1.ScrollGraph;
//How this done seems to need own tut, at 29 Jan 16
var SrcRect,DestRect:TRect;
begin
  SrcRect:=Rect(1,0,wImageW+1,wImageH);
  DestRect:=Rect(0,0,wImageWm1,wImageH);
  Image1.Picture.Bitmap.canvas.CopyRect(DestRect,Image1.Picture.Bitmap.canvas,SrcRect);
  DrawLine(clWhite,wImageWm1,0,wImageWm1,wImageH);
end;//ScrollGraph;

procedure Tsim07f1.DrawLine(clL:TColor;wXL0,wYL0,wXL1,wYL1:word);
begin
  Image1.Picture.Bitmap.canvas.pen.color:=clL;
  Image1.Picture.Bitmap.canvas.moveto(wXL0,wYL0);
  Image1.Picture.Bitmap.canvas.lineto(wXL1,wYL1);
end;//DrawDiag;

procedure Tsim07f1.DrawDiag;
begin
  DrawLine(clLime,10,10,wImageW-10,wImageH-10);
end;//DrawDiag;

procedure Tsim07f1.PlotPoints;
begin
  PlotPoint(clGrass,wImageWm1,wScGrass,wOffGrass,lwBM_grass);//Grass
  PlotPoint(clRabbit,wImageWm1,wScRabbit,wOffRabbit,lwBM_rabbit);//Rabbit
  PlotPoint(clFox,wImageWm1,wScFox,wOffFox,lwBM_fox);//Fox

end;//PlotPoints

procedure Tsim07f1.PlotPoint(clColorL:TColor;
                 wXL,wScaleY,wOffsetY:word;lwToPlotYL:longword);
(*Plots a point, in color supplied. The X cood will be wXL (in the
   coord system of the drawing surface. The Y cood will be derived
   from lwToPlot, scaled and offset by the numbers passed.
   The line will wrap, vertically.*)
function wScaleIt(lwToPlotYL2:longword;
                   wScaleY,wOffsetY:word):word;//SR of PlotPoint
//Bad practice: Uses a global inside it. (wImageH)

(*Returns an acceptable Y cood value, in the range 0-wImageH(-1?...
  whatever is needed!), starting from the value to be plotted,
  supplied in lwBM_grass:=lwToPlotYL2, and first "squashes" or "stretches",
  depending on wScaleY, and then offsets by wOffsetY.
The scale factor: If equal to 100,000, doesn't squash or stretch. Less than
  100,000, squashes. More than 100,000, stretches.
  50,000 halves each value. 200,000 doubles each value.*)
var lwTmp:longword;

  begin
    //First do the "squashing" or "stretching"
    lwTmp:=lwToPlotYL2;
    lwTmp:=(lwTmp*wScaleY) div 100000;
    //Now do offset and wrap...
    lwTmp:=(lwTmp+wOffsetY) mod wImageH;
    result:=lwTmp;
  end;//ScaleIt;

begin
  Image1.Picture.Bitmap.canvas.pixels[wImageWm1,
                wImageH-(wScaleIt(lwToPlotYL,wScaleY,wOffsetY))]:=clColorL;
end;//PlotPoint
end.
